package options

import (
	"time"

	commonpb "git.perx.ru/perxis/perxis-go/proto/common"
)

// SortOptions настройки сортировки результатов
type SortOptions struct {
	Sort []string
}

// PaginationOptions настройки возвращаемых страниц результатов
type PaginationOptions struct {
	Limit  int
	Offset int
}

// FieldOptions настройки включения/исключения полей из результатов запроса
type FieldOptions struct {
	// Fields - Наименования полей для включения/исключения из результатов запроса (только указанные поля)
	// Если `ExcludeFields` не установлен, то результат содержит только указанные поля
	// Если `ExcludeFields` установлен, то результат содержит все поля кроме указанных
	Fields []string

	// ExcludeFields- Если флаг установлен, то перечисленные поля `Fields` следует исключить из результатов
	ExcludeFields bool
}

// FindOptions настройки возвращаемых результатов поиска
type FindOptions struct {
	SortOptions
	PaginationOptions
	FieldOptions
}

// Deprecated использовать New
// NewFindOptions создает новые результаты поиска
func NewFindOptions(pageNum, pageSize int, sort ...string) *FindOptions {
	return New(pageNum*pageSize, pageSize, sort...)
}

// New создает новые параметры поиска
func New(offset, limit int, sort ...string) *FindOptions {
	return &FindOptions{
		PaginationOptions: PaginationOptions{
			Offset: offset,
			Limit:  limit,
		},
		SortOptions: SortOptions{
			Sort: sort,
		},
	}
}

// MergeFindOptions объединяет в FindOptions различные варианты настроек
func MergeFindOptions(opts ...interface{}) *FindOptions {
	fo := &FindOptions{}
	for _, opt := range opts {
		if opt == nil {
			continue
		}

		switch o := opt.(type) {
		case FindOptions:
			fo.SortOptions = MergeSortOptions(fo.SortOptions, o.SortOptions)
			fo.PaginationOptions = MergePaginationOptions(fo.PaginationOptions, o.PaginationOptions)
			fo.FieldOptions = MergeFieldOptions(fo.FieldOptions, o.FieldOptions)
		case *FindOptions:
			fo.SortOptions = MergeSortOptions(fo.SortOptions, o.SortOptions)
			fo.PaginationOptions = MergePaginationOptions(fo.PaginationOptions, o.PaginationOptions)
			fo.FieldOptions = MergeFieldOptions(fo.FieldOptions, o.FieldOptions)
		case SortOptions:
			fo.SortOptions = MergeSortOptions(fo.SortOptions, o)
		case *SortOptions:
			fo.SortOptions = MergeSortOptions(fo.SortOptions, *o)
		case PaginationOptions:
			fo.PaginationOptions = MergePaginationOptions(fo.PaginationOptions, o)
		case *PaginationOptions:
			fo.PaginationOptions = MergePaginationOptions(fo.PaginationOptions, *o)
		case FieldOptions:
			fo.FieldOptions = o
		case *FieldOptions:
			fo.FieldOptions = *o
		}
	}
	return fo
}

func FindOptionsFromPB(protoOpts *commonpb.FindOptions) *FindOptions {
	if protoOpts == nil {
		return nil
	}
	fo := &FindOptions{
		SortOptions: SortOptions{
			Sort: protoOpts.Sort,
		},
		PaginationOptions: PaginationOptions{
			Limit:    int(protoOpts.Limit),
			Offset:   int(protoOpts.Offset),
		},
		FieldOptions: FieldOptions{
			Fields:        protoOpts.Fields,
			ExcludeFields: protoOpts.ExcludeFields,
		},
	}

	if fo.Offset == 0 && fo.Limit == 0 {
		fo.Offset = int(protoOpts.PageSize * protoOpts.PageNum)
		fo.Limit = int(protoOpts.PageSize)
	}

	return fo
}

func FindOptionsToPB(opts *FindOptions) *commonpb.FindOptions {
	if opts == nil {
		return nil
	}
	fo := &commonpb.FindOptions{
		Sort:          opts.Sort,
		Offset:        int32(opts.Offset),
		Limit:         int32(opts.Limit),
		Fields:        opts.Fields,
		ExcludeFields: opts.ExcludeFields,
	}
	return fo
}

type TimeFilter struct {
	Before, After time.Time
}

// MergeSortOptions объединяет настройки сортировки
func MergeSortOptions(options ...SortOptions) SortOptions {
	fo := SortOptions{}
	for _, opt := range options {
		if len(opt.Sort) == 0 {
			continue
		}
		fo.Sort = append(fo.Sort, opt.Sort...)
	}
	return fo
}

// MergePaginationOptions объединяет настройки страниц
func MergePaginationOptions(options ...PaginationOptions) PaginationOptions {
	fo := PaginationOptions{}
	for _, opt := range options {
		if opt.Offset == 0 && opt.Limit == 0 {
			continue
		}
		fo.Offset = opt.Offset
		fo.Limit = opt.Limit
	}
	return fo
}

// MergeFieldOptions выполняет слияние опций для возвращаемых полей.
// Выбирается не пустое значение.
func MergeFieldOptions(options ...FieldOptions) FieldOptions {
	fo := FieldOptions{}
	for _, opt := range options {
		if len(opt.Fields) > 0 {
			fo.Fields = opt.Fields
			fo.ExcludeFields = opt.ExcludeFields
			return fo
		}
	}
	return fo
}
